note
	description: "This class simulates a forest fire."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	FOREST_FIRE

create
	make

feature {NONE} -- creation

	make
			-- Create a forest fire.
		do
			create map.make_filled (Empty, Height, Width)
		end

feature -- access

	map: ARRAY2 [INTEGER]
			-- Map of the forest

	Width: INTEGER = 15

	Height: INTEGER = 15

feature -- action

	do_step
			-- Calculate the next state of the forest.
		local
			row, col: INTEGER
			map_copy: ARRAY2 [INTEGER]
		do
				-- TODO: Implement this.
			from
				create map_copy.make_filled (Empty, map.width, map.height)
				map_copy.copy (map)
				row := 1
			until
				map_copy.height < row
			loop
				from
					col := 1
				until
					map_copy.width < col
				loop
					inspect map_copy [row, col]
					when Empty then
							-- Spawn a tree with probability `Spawn_probability'.
						random.forth
						if random.unfair_bit (Spawn_probability) then
							map_copy [row, col] := Tree
						end
					when Tree then
							-- Ignite a tree if a neighbor is burning or else with
							-- probability `Ignite_probability'.

						if neighbor_is_on_fire (row, col) then
							map_copy [row, col] := Fire
						else
							random.forth
							if random.unfair_bit (Ignite_probability) then
								map_copy [row, col] := Fire
							end
						end
					when Fire then
							-- Reset the tile to be empty again.
						map_copy [row, col] := Empty
					else
						check
							map_contains_invalid_element: false
						end
					end
					col := col + 1
				end
				row := row + 1
			end
			map := map_copy
		end

feature {NONE} -- implementation

	random: V_RANDOM
			-- Random sequence for spawning/igniting trees.
		once
			create Result
		end

	neighbor_is_on_fire (row, column: INTEGER): BOOLEAN
			-- Is any of the neighboring tiles on fire?
		local
			row_delta, column_delta: INTEGER
		do
				-- TODO: Implement this.
			from
				row_delta := -1
			until
				1 < row_delta or else result
			loop
				from
					column_delta := -1
				until
					1 < column_delta or else result
				loop
					result := result
					          or else (
					            (row_delta /= 0 or column_delta /= 0)
					            and then
					            (0 < row + row_delta and row + row_delta <= map.height)
					            and then
					            (0 < column + column_delta and column + column_delta <= map.width)
					            and then
					            map [row + row_delta, column + column_delta] = Fire)
					column_delta := column_delta + 1
				end
				row_delta := row_delta + 1
			end
		end

feature -- constants

	Ignite_probability: REAL = 0.2
			-- the probability that a tree just ignites even if no neighbor is burning

	Spawn_probability: REAL = 0.4
			-- the probability that new tree spawns in an empty tile

	Empty: INTEGER = 0

	Tree: INTEGER = 1

	Fire: INTEGER = 2

invariant
	map /= Void
	map.height = Height and map.width = Width

end
